Noncomp.bayesMixed <- function(formulae, Z, D, grp, data = parent.frame(),
                               n.draws = 5000, param = TRUE,
                               in.sample = FALSE, model.c = "probit",
                               model.o = "probit",
                               tune.fixed = 0.01, tune.random = 0.01, 
                               p.mean.c = 0, p.prec.c = 0.01, p.mean.o = 0,
                               p.prec.o = 0.01, p.mean.r = 0, p.prec.r = 0.01,
                               p.df.var = 10, p.scale.var = 1,
                               mda.probit = TRUE,
                               coef.start.c = 0, coef.start.o = 0,
                               coef.start.r = 0, var.start.o = 1,
                               Psi.start.c = 1,
                               Psi.start.o = 1, Psi.start.r = 1,
                               p.df.c = 5, p.df.o = 5, p.df.r = 5,
                               p.scale.c = 1, p.scale.o = 1,
                               p.scale.r = 1,
                               burnin = 0, thin = 0, verbose = TRUE) {  
  
  ## models
  if (!(model.o %in% c("probit", "gaussian")))
    stop("no such model is supported as the outcome model")
  if (!(model.c %in% c("probit", "logit")))
    stop("no such model is supported as the compliance model")
  if (model.c == "logit") 
    logit.c <- TRUE
  else
    logit.c <- FALSE
  
  ## getting the data
  call <- match.call()
  ## outcome model: fixed effects
  mf <- model.frame(formulae[[1]], data=data, na.action='na.pass')
  Xo <- model.matrix(formulae[[1]], data=mf)
  if (sum(is.na(Xo)) > 0)
    stop("missing values not allowed in covariates")
  if (model.o == "probit")
    Y <- as.integer(model.response(mf))
  ## outcome model: random effects
  Wo <- model.matrix(formulae[[2]], data=data)

  ## compliance model
  mf <- model.frame(formulae[[3]], data=data, na.action='na.fail')
  Xc <- model.matrix(formulae[[3]], data=mf)
  Wc <- model.matrix(formulae[[4]], data=data)
  
  ## response model
  mf <- model.frame(formulae[[5]], data=data, na.action='na.pass')
  Xr <- model.matrix(formulae[[5]], data=mf)
  Wr <- model.matrix(formulae[[6]], data=data)

  ## other variables 
  N <- length(Y)
  Z <- eval(call$Z, envir = data)
  D <- eval(call$D, envir = data)
  grp <- eval(call$grp, envir = data)
  ngrp <- length(table(grp))
  if (sum(is.na(Z)) > 0)
    stop("missing values not allowed in the encouragement variable")
  if (sum(is.na(D)) > 0)
    stop("missing values not allowed in the treatment variable")
  if (sum(is.na(grp)) > 0)
    stop("missing values not allowed in the group variable")
  
  ## Random starting values for missing Y using Bernoulli(0.5)
  R <- (!is.na(Y))*1
  NR <- is.na(Y)
  Ymiss <- sum(NR)
  if (Ymiss > 0)
    if (model.o == "probit")
      Y[NR] <- (runif(Ymiss) > 0.5)*1
    else
      Y[NR] <- rnorm(Ymiss)
  
  ## Compliance status: 0 = noncomplier, 1 = complier
  C <- rep(NA, N)
  C[Z == 1 &  D == 0] <- 0 # never-takers

  ## Always-takers: 0 = never-takers/compliers, 1 = always-takers
  if (sum(Z == 0 & D == 1)>0) { # some always-takers
    AT <- TRUE
    A <- rep(NA, N)
    A[D == 0] <- 0            # never-takers or compliers
    A[Z == 0 & D == 1] <- 1  
    if (logit.c)
      C[Z == 0 & D == 1] <- 2 # always-takers
    else
      C[Z == 0 & D == 1] <- 0 # always-takers
  } else { # no always-takers
    A <- rep(0, N)
    AT <- FALSE
    C[Z == 1 & D == 1] <- 1 # compliers
  }
  res <- list(call = call, Y = Y, R = R, Xo = Xo, Xc = Xc, Xr = Xr,
              Zo = Wo, Zc = Wc, Zr = Wr, A = A, C = C, D = D, Z = Z,
              grp = grp, n.draws = n.draws)
  
  ## Random starting values for missing compliance status
  if (AT) {
    A[is.na(A)] <- (runif(sum(is.na(A))) > 0.5)*1
    if (logit.c)
      C[A == 1] <- 2
    else
      C[A == 1] <- 0
  }
  C[is.na(C)] <- (runif(sum(is.na(C))) > 0.5)*1
  
  ## Completing the outcome model matrix 
  ## The default category is never-takers
  X <- Xo
  W <- Wo
  X1 <- Xr
  W1 <- Wr
  if (AT) { # when some always-takers exist
    ## X and W include an intercept
    ## Xo = [c1 c0 a X] where c1 for compliers with encouragement
    ##                        c0 for compliers without encouragement
    ##                        a for always-takers with/without encouragement  
    ## Wo = [c a W] - no Z for parsimony
    Xo <- cbind(0, 0, 0, X)
    Wo <- cbind(0, 0, W)
    Xr <- cbind(0, 0, 0, X1)
    Wr <- cbind(0, 0, W1)
    Xo[A == 1, 3] <- Wo[A == 1, 2] <- 1
    Xr[A == 1, 3] <- Wr[A == 1, 2] <- 1
    colnames(Xo) <- c("Complier1", "Complier0", "AlwaysTaker",
                      colnames(X))
    colnames(Wo) <- c("Complier", "AlwaysTaker", colnames(W))
    colnames(Xr) <- c("Complier1", "Complier0", "AlwaysTaker",
                      colnames(X1))
    colnames(Wr) <- c("Complier", "AlwaysTaker", colnames(W1))
  } else { # when always-takers do not exist
    ## Xo = [c1 c0 X] where c1 for compliers with encouragement
    ##                      c0 for compliers without encouragement
    ## Wo = [c W] for parsimony
    Xo <- cbind(0, 0, X)
    Wo <- cbind(0, W)
    Xr <- cbind(0, 0, X1)
    Wr <- cbind(0, W1)
    colnames(Xo) <- c("Complier1", "Complier0", colnames(X))
    colnames(Wo) <- c("Complier", colnames(W))
    colnames(Xr) <- c("Complier1", "Complier0", colnames(X1))
    colnames(Wr) <- c("Complier", colnames(W1))
  }
  Wo[C == 1, 1] <- Wr[C == 1, 1] <- 1
  Xo[C == 1 & Z == 1, 1] <- Xr[C == 1 & Z == 1, 1] <- 1
  Xo[C == 1 & Z == 0, 2] <- Xr[C == 1 & Z == 0, 2] <- 1
  
  ## dimensions
  nfixedC <- ncol(Xc)
  nfixedO <- ncol(Xo)
  nfixedR <- ncol(Xr)
  nrandomC <- ncol(Wc)
  nrandomO <- ncol(Wo)
  nrandomR <- ncol(Wr)
  if (AT)
    nqoi <- 8
  else
    nqoi <- 7
  
  ## checking starting values and prior for fixed effects
  if (logit.c & AT) {
    if(length(p.mean.c) != nfixedC*2)
      if (length(p.mean.c) == 1)
        p.mean.c <- rep(p.mean.c, nfixedC*2)
      else
        stop(paste("the length of p.mean.c should be", nfixedC*2))    
    if(length(coef.start.c) != nfixedC*2)
      if (length(coef.start.c) == 1)
        coef.start.c <- rep(coef.start.c, nfixedC*2)
      else
        stop(paste("the length of coef.start.c should be", nfixedC*2))        
  } else {
    if(length(p.mean.c) != nfixedC)
      if (length(p.mean.c) == 1)
        p.mean.c <- rep(p.mean.c, nfixedC)
      else
        stop(paste("the length of p.mean.c should be", nfixedC))        
    if(length(coef.start.c) != nfixedC)
      if (length(coef.start.c) == 1)
        coef.start.c <- rep(coef.start.c, nfixedC)
      else
        stop(paste("the length of coef.start.c should be", nfixedC))    
  }

  if(length(coef.start.o) != nfixedO)
    if (length(coef.start.o) == 1)
      coef.start.o <- rep(coef.start.o, nfixedO)
    else
      stop(paste("the length of coef.start.o should be", nfixedO))      
  if(length(p.mean.o) != nfixedO)
    if (length(p.mean.o) == 1)
      p.mean.o <- rep(p.mean.o, nfixedO)
    else
      stop(paste("the length of p.mean.o should be", nfixedO))    

  if(length(coef.start.r) != nfixedR)
    if (length(coef.start.r) == 1)
      coef.start.r <- rep(coef.start.r, nfixedR)
    else
      stop(paste("the length of coef.start.r should be", nfixedR))    
  if(length(p.mean.r) != nfixedR)
    if (length(p.mean.r) == 1)
      p.mean.r <- rep(p.mean.r, nfixedR)
    else
      stop(paste("the length of p.mean.r should be", nfixedR))    

  if(is.matrix(p.prec.c)) {
    if (dim(p.prec.c) != rep(nfixedC*2, 2))
        stop(paste("the dimension of p.prec.c should be",
                   rep(nfixedC*2, 2)))    
  } else if (length(p.prec.c) == 1){
    if (logit.c & AT)
      p.prec.c <- diag(p.prec.c, nfixedC*2)
    else
      p.prec.c <- diag(p.prec.c, nfixedC)
  } else {
    stop("Incorrect input for p.prec.c")
  }

  if(is.matrix(p.prec.o)) {
    if (dim(p.prec.o) != rep(nfixedO, 2))
      stop(paste("the dimension of p.prec.o should be",
                 rep(nfixedO, 2)))    
  } else if (length(p.prec.o) == 1){
    p.prec.o <- diag(p.prec.o, nfixedO)
  } else {
    stop("Incorrect input for p.prec.o")
  }

  if(is.matrix(p.prec.r)) {
    if (dim(p.prec.r) != rep(nfixedR, 2))
      stop(paste("the dimension of p.prec.r should be",
                 rep(nfixedR, 2)))    
  } else if (length(p.prec.r) == 1){
    p.prec.r <- diag(p.prec.r, nfixedR)
  } else {
    stop("Incorrect input for p.prec.r")
  }

  ## starting values for Psi
  if(is.matrix(Psi.start.c)) {
    if (dim(Psi.start.c) != rep(nrandomC, 2))
      stop(paste("the dimension of Psi.start.c should be",
                 rep(nrandomC, 2)))    
  } else if (length(Psi.start.c) == 1){
    Psi.start.c <- diag(Psi.start.c, nrandomC)
  } else {
    stop("Incorrect input for Psi.start.c")
  }

  if(is.matrix(Psi.start.o)) {
    if (dim(Psi.start.o) != rep(nrandomO, 2))
      stop(paste("the dimension of Psi.start.o should be",
                 rep(nrandomO, 2)))    
  } else if (length(Psi.start.o) == 1){
    Psi.start.o <- diag(Psi.start.o, nrandomO)
  } else {
    stop("Incorrect input for Psi.start.o")
  }

  if(is.matrix(Psi.start.r)) {
    if (dim(Psi.start.r) != rep(nrandomR, 2))
      stop(paste("the dimension of Psi.start.r should be",
                 rep(nrandomR, 2)))    
  } else if (length(Psi.start.r) == 1){
    Psi.start.r <- diag(Psi.start.r, nrandomR)
  } else {
    stop("Incorrect input for Psi.start.r")
  }
  
  ## prior scale matrix for Psi's
  if(is.matrix(p.scale.c)) {
    if (dim(p.scale.c) != rep(nrandomC, 2))
        stop(paste("the dimension of p.scale.c should be",
                   rep(nrandomC, 2)))    
  } else if (length(p.scale.c) == 1)
    p.scale.c <- diag(p.scale.c, nrandomC)
  else 
    stop("Incorrect input for p.scale.c")
  
  if(is.matrix(p.scale.o)) {
    if (dim(p.scale.o) != rep(nrandomO, 2))
        stop(paste("the dimension of p.scale.o should be",
                   rep(nrandomO, 2)))    
  } else if (length(p.scale.o) == 1)
    p.scale.o <- diag(p.scale.o, nrandomO)
  else 
    stop("Incorrect input for p.scale.o")

  if(is.matrix(p.scale.r)) {
    if (dim(p.scale.r) != rep(nrandomR, 2))
        stop(paste("the dimension of p.scale.r should be",
                   rep(nrandomR, 2)))    
  } else if (length(p.scale.r) == 1)
    p.scale.r <- diag(p.scale.r, nrandomR)
  else 
   stop("Incorrect input for p.scale.r")
  
  
  ## proposal variance for logits
  if (AT) {
    if (length(tune.fixed) != nfixedC*2)
      if (length(tune.fixed) == 1)
        tune.fixed <- rep(tune.fixed, nfixedC*2)
      else
        stop(paste("the length of tune.fixed should be", nfixedC*2))
    if (length(tune.random) != 2)
      if (length(tune.random) == 1)
        tune.random <- rep(tune.random, 2)
      else
        stop("the length of tune.random should be 2")
  } else {
    if (length(tune.fixed) != nfixedC)
      if (length(tune.fixed) == 1)
        tune.fixed <- rep(tune.fixed, nfixedC)
      else
        stop(paste("the length of tune.fixed should be", nfixedC))
    if (length(tune.random) != 1)
      stop("the length of tune.random should be 1")
  }
  
  ## checking thinnig and burnin intervals
  if (n.draws <= 0)
    stop("`n.draws' should be a positive integer.")
  if (burnin < 0 || burnin >= n.draws)
    stop("`burnin' should be a non-negative integer less than `n.draws'.")
  if (thin < 0 || thin >= n.draws)
    stop("`thin' should be a non-negative integer less than `n.draws'.")
  keep <- thin + 1

  ## calling C function
  if (model.o == "probit") 
    out <- .C("LIbprobitMixed",
              as.integer(Y), as.integer(R), as.integer(Z),
              as.integer(D), as.integer(C), as.integer(A),
              as.integer(grp), as.integer(Ymiss), as.integer(AT),
              as.integer(in.sample), as.double(Xc), as.double(Wc),
              as.double(Xo), as.double(Wo), as.double(Xr),
              as.double(Wr), as.double(coef.start.c),
              as.double(coef.start.c), as.double(coef.start.o), as.double(coef.start.r),
              as.integer(N), as.integer(n.draws), as.integer(ngrp),
              as.integer(max(table(grp))),
              as.integer(c(nfixedC,nfixedO,nfixedR)),
              as.integer(c(nrandomC, nrandomO, nrandomR)),
              as.double(Psi.start.c), as.double(Psi.start.c),
              as.double(Psi.start.o), as.double(Psi.start.r),
              as.double(p.mean.c), as.double(p.mean.o), as.double(p.mean.r),
              as.double(p.prec.c), as.double(p.prec.o), as.double(p.prec.r),
              as.integer(c(p.df.c,p.df.c,p.df.o,p.df.r)),
              as.double(p.scale.c), as.double(p.scale.c),
              as.double(p.scale.o), as.double(p.scale.r),
              as.double(tune.fixed), as.double(tune.random), as.integer(logit.c),
              as.integer(param), as.integer(mda.probit), as.integer(burnin),
              as.integer(keep), as.integer(verbose),
              coefC = double(nfixedC*(ceiling((n.draws-burnin)/keep))),
              coefA = double(nfixedC*(ceiling((n.draws-burnin)/keep))),
              coefO = double(nfixedO*(ceiling((n.draws-burnin)/keep))),
              coefR = double(nfixedR*(ceiling((n.draws-burnin)/keep))),
              sPsiC = double(nrandomC*(nrandomC+1)*(ceiling((n.draws-burnin)/keep))/2),
              sPsiA = double(nrandomC*(nrandomC+1)*(ceiling((n.draws-burnin)/keep))/2),
              sPsiO = double(nrandomO*(nrandomO+1)*(ceiling((n.draws-burnin)/keep))/2),
              sPsiR = double(nrandomR*(nrandomR+1)*(ceiling((n.draws-burnin)/keep))/2),
              QoI = double(nqoi*(ceiling((n.draws-burnin)/keep))),
              PACKAGE = "experiment")
  else
    out <- .C("LINormalMixed",
              as.double(Y), as.integer(R), as.integer(Z),
              as.integer(D), as.integer(C), as.integer(A),
              as.integer(grp), as.integer(Ymiss), as.integer(AT),
              as.integer(in.sample), as.double(Xc), as.double(Wc),
              as.double(Xo), as.double(Wo), as.double(Xr),
              as.double(Wr), as.double(coef.start.c),
              as.double(coef.start.c), as.double(coef.start.o),
              as.double(coef.start.r), as.double(var.start.o),
              as.integer(N), as.integer(n.draws), as.integer(ngrp),
              as.integer(max(table(grp))),
              as.integer(c(nfixedC,nfixedO,nfixedR)),
              as.integer(c(nrandomC, nrandomO, nrandomR)),
              as.double(Psi.start.c), as.double(Psi.start.c),
              as.double(Psi.start.o), as.double(Psi.start.r),
              as.double(p.mean.c), as.double(p.mean.o), as.double(p.mean.r),
              as.double(p.prec.c), as.double(p.prec.o),
              as.double(p.prec.r),
              as.integer(p.df.var), as.double(p.scale.var),
              as.integer(c(p.df.c,p.df.c,p.df.o,p.df.r)),
              as.double(p.scale.c), as.double(p.scale.c),
              as.double(p.scale.o), as.double(p.scale.r),
              as.double(tune.fixed), as.double(tune.random), as.integer(logit.c),
              as.integer(param), as.integer(mda.probit), as.integer(burnin),
              as.integer(keep), as.integer(verbose),
              coefC = double(nfixedC*(ceiling((n.draws-burnin)/keep))),
              coefA = double(nfixedC*(ceiling((n.draws-burnin)/keep))),
              coefO = double(nfixedO*(ceiling((n.draws-burnin)/keep))),
              coefR = double(nfixedR*(ceiling((n.draws-burnin)/keep))),
              ssig2 = double(ceiling((n.draws-burnin)/keep)),
              sPsiC = double(nrandomC*(nrandomC+1)*(ceiling((n.draws-burnin)/keep))/2),
              sPsiA = double(nrandomC*(nrandomC+1)*(ceiling((n.draws-burnin)/keep))/2),
              sPsiO = double(nrandomO*(nrandomO+1)*(ceiling((n.draws-burnin)/keep))/2),
              sPsiR = double(nrandomR*(nrandomR+1)*(ceiling((n.draws-burnin)/keep))/2),
              QoI = double(nqoi*(ceiling((n.draws-burnin)/keep))),
              PACKAGE = "experiment")
    
  if (param) {
    res$coefC <- matrix(out$coefC, byrow = TRUE, ncol = nfixedC)
    colnames(res$coefC) <- colnames(Xc)
    res$PsiC <- matrix(out$sPsiC, byrow = TRUE, ncol = nrandomC*(nrandomC+1)/2)
    if (AT) {
      res$coefA <- matrix(out$coefA, byrow = TRUE, ncol = nfixedC)
      colnames(res$coefA) <- colnames(Xc)
      res$PsiA <- matrix(out$sPsiA, byrow = TRUE, ncol = nrandomC*(nrandomC+1)/2)
    }
    res$coefO <- matrix(out$coefO, byrow = TRUE, ncol = nfixedO)
    res$PsiO <- matrix(out$sPsiO, byrow = TRUE, ncol = nrandomO*(nrandomO+1)/2)
    colnames(res$coefO) <- colnames(Xo)
    if (Ymiss > 0) {
      res$coefR <- matrix(out$coefR, byrow = TRUE, ncol = nfixedR)
      res$PsiR <- matrix(out$sPsiR, byrow = TRUE, ncol = nrandomR*(nrandomR+1)/2)
      colnames(res$coefR) <- colnames(Xr)
    }
    if (model.o == "gaussian")
      res$sig2 <- out$ssig2
  }
  QoI <- matrix(out$QoI, byrow = TRUE, ncol = nqoi)
  res$ITT <- QoI[,1]
  res$CACE <- QoI[,2]
  res$pC <- QoI[,3]
  res$pN <- QoI[,4]
  if (AT)
    res$pA <- 1-QoI[,3]-QoI[,4]
  res$Y1barC <- QoI[,5]
  res$Y0barC <- QoI[,6]
  res$YbarN <- QoI[,7]
  if (AT) 
    res$YbarA <- QoI[,8]

  return(res)
}
